#ifndef lint
static char *RCSid = "$Header: /sequent2/empire/EMP/player/commands/RCS/sell.c,v 1.6 89/09/10 19:22:40 mr-frog Exp $";
#endif

/*
 * sell.c
 *
 * engage in inter-nation trading of commodities.
 * Place commodities on-sale at the deity trading post.
 *
 * Dave Pare, 1986
 */

#include "misc.h"
#include "xy.h"
#include "file.h"
#include "var.h"
#include "sect.h"
#include "item.h"
#include "nsc.h"
#include "nat.h"
#include "nuke.h"
#include "plane.h"
#include "ship.h"
#include <math.h>		/* bailey@math-cs.kent.edu */
#include "commodity.h"
#include "land.h"
#include "trade.h"
#include "player.h"
/*#define EF_COMM 10*/

/*
 * format: sell <COMMODITY> <SECTS> <NUMBER> <PRICE>
 *   where NUMBER represents either the number to reach
 *   or, if negative, the abs number to try and get from
 *   each sector.
 */
sell()
{
	extern char *argp[];
	struct sctstr	sect;
	struct nstr_sect nstr;
	struct ichrstr	*ip;
	struct comstr comm;
	int     number_set;
	int     number_sub;
	int     totalcom;
	int     amt;
	int     com;
	char	*p;
	float   price;
	char	cc;
        time_t	now;
	int	ii = 0;
        coord   x, y;
	s_char  buf[1024];

	check_market();
	check_trade();
	if ((ip = whatitem(player->argp[1], "Commodity you want to sell: ")) == 0)
		return RET_SYN;
	if (ip->i_sell == 0) {
		pr("You can't sell %s\n", ip->i_name);
		return RET_FAIL;
	}
	if (!(p = getstarg(player->argp[2], "Sector to sell from: ",buf)))
	    return RET_SYN;
	if (!sarg_xy(p, &x, &y))
	    return RET_SYN;
	if (!getsect(x, y, &sect))
	    pr("Could not access that sector.\n");
	if (sect.sct_type != SCT_TRADE || !player->owner){
	    pr("That sector is not a trading post\n");
	    return RET_FAIL;
	}
	if(sect.sct_effic < 60){
	    pr("Trading posts need to be >= 60%% efficient to sell goods.\n");
	    return RET_FAIL;
	}
	number_set = 0;
	number_sub = 0;
	if ((p = getstarg(player->argp[3], "Amount:  ",buf)) == 0 || *p == 0)
		return RET_SYN;
	number_set = atoi(p);
	if ((p = getstarg(player->argp[4], "Price per unit: ", buf)) == 0 ||
	    *p == 0)
		return RET_SYN;
	price = atof(p);
	if (price <= 0.0) {
		pr("No sale.\n");
		return RET_FAIL;
	}
	if (price > 1000.0) /* Inf can cause overflow */
		price = 1000.0;		  /* bailey@math-cs.kent.edu */
	totalcom = 0;
	/*
	 * military control necessary to sell
	 * goodies in occupied territory.
	 */
	if (sect.sct_oldown != player->cnum) {
	    int     tot_mil=0;
	    struct  nstr_item ni;
	    struct  lndstr land;

	    snxtitem_xy(&ni, EF_LAND, sect.sct_x, sect.sct_y);
	    while (nxtitem(&ni, (s_char *)&land)){
		if (land.lnd_own == player->cnum)
		    tot_mil  += total_mil(&land);
	    }
	    if (tot_mil+(getvar(V_MILIT, (char *)&sect, EF_SECTOR)) * 10
		< getvar(V_CIVIL, (char *)&sect, EF_SECTOR)) {
		pr("Military control required to sell goods.\n");
		return RET_FAIL;
	    }
	}
	if (((amt = getvar(ip->i_vtype, (char *)&sect,EF_SECTOR))) == 0) {
	    pr("You don't have any %s to sell there.\n", ip->i_name);
	    return RET_FAIL;
	}
	if (number_set > 0)
	    com = min(number_set, amt);
	else if (number_set < 0)
	    com = amt + number_set;
	else
	    com = 0;
	if (com <= 0)
	    return RET_SYN;
	totalcom += com;
	amt -= com;
	pr("Sold %d %s at %s (%d left)\n", com, ip->i_name,
	   xyas(sect.sct_x, sect.sct_y, player->cnum), amt);
	putvar(ip->i_vtype, amt, (char *)&sect, EF_SECTOR);
	cc = ip->i_mnem;
	putsect(&sect);
	if (totalcom > 0) {
	    for (ii = 0; getcomm(ii, &comm); ii++) {
		if (comm.com_owner == 0)
		    break;
	    }
	    if (getcomm(ii, &comm) == 0)
		ef_extend(EF_COMM, 1);
	    (void) time(&now);
	    comm.com_type = ip->i_mnem;
	    comm.com_owner = player->cnum;
	    comm.com_price = price;
	    comm.com_maxbidder = player->cnum;
	    comm.com_maxprice = price;
	    comm.com_markettime = now;
	    comm.com_amount = totalcom;
	    comm.com_x = 0;
	    comm.com_y = 0;
	    comm.sell_x = sect.sct_x;
	    comm.sell_y = sect.sct_y;
	    if(!putcomm(ii, &comm)) {
		pr("Problems with the commodities file, call the Deity\n");
		return RET_FAIL;
	    }
	} else {
	    pr("No eligible %s for sale\n", ip->i_name);
	    return RET_FAIL;
	}
	return RET_OK;
}


