#include <assert.h>
#include <stdio.h>
#include <ctype.h>
#include <malloc.h>
#include "misc.h"
#include "file.h"
#include "xy.h"
#include "nsc.h"
#include "genitem.h"
#include "edb.h"
#include "edbproc.h"	/* public  structures */
#include "edbpriv.h"	/* private structures */


#define MAXTYPE 8
char *typenames[] = {
	"bad",
	"char *",
	"u_char",
	"short",
	"u_short",
	"int",
	"long",
	"float",
	"double" };
	
char *nscnames[] = {
	"bad",
	"NSC_CHARP",
	"NSC_UCHAR",
	"NSC_SHORT",
	"NSC_USHORT",
	"NSC_INT",
	"NSC_LONG",
	"NSC_FLOAT",
	"NSC_FLOAT" };
	
char *tableName(dict)
	Dictionary *dict;
{
	if (dict)
		if (dict->objdesc)
			return dict->objdesc->objdescr;
	return "unknown";
}

char *nscTableName(dict)
	Dictionary *dict;
{
	static char buf[80];
	if (dict) {
		if (dict->objdesc) {
			return dict->objdesc->nscname;
		}
	}
	sprintf(buf, "Object_type_%d", dict->objnum);
	return buf;
}

char *prefixName(dict)
	Dictionary *dict;
{
	if (dict) {
		if (dict->objdesc) {
			return dict->objdesc->fldprefix;
		}
	}
	return "fld_";
}

char *strTableName(dict)
	Dictionary *dict;
{
	static char buf[80];
	if (dict)
		if (dict->objdesc) {
			return dict->objdesc->strname;
		}
	sprintf(buf, "Object_type_%d_ca", dict->objnum);
	return buf;
}


void describeType(outf, dict)
	FILE *outf;
	Dictionary *dict;
{
	Field *f;
	char *typename;
	char *strname;
	char *prefix;
	char fullname[32];
	prefix = prefixName(dict);
	strname = strTableName(dict);
	fprintf(outf, "/* Structure definition for %s */\n",
		tableName(dict) );
	fprintf(outf, "struct %s { /* %d */\n", 
		strname, dict->objnum);
	fprintf(outf, "	short ef_type;\n");
	for (f=dict->flds;f;f=f->next) {
		if (f->fldtype>0 && f->fldtype<=MAXTYPE)
			typename = typenames[f->fldtype];
		else
			typename = typenames[0];
		strcpy(fullname, f->fldname);
		if (f->fldcount!=0&&f->fldtype!=EDB_CHAR)
			sprintf(fullname,"%s[%d]", f->fldname, f->fldcount);
		fprintf(outf, "	%-10s %s%-16s; /* %d */\n", 
			typename, prefix, fullname, f->fldtype);
	}
	fprintf(outf,"\n#"
		     "ifdef %s_EXTRA\n"
		     "	%s_EXTRA\n#"
		     "endif\n\n", strname, strname);
	fprintf(outf, "};\n\n");
}

void makeNscDefinition(outf, dict)
	FILE *outf;
	Dictionary *dict;
{
	Field *f;
	char *typename;
	char *strname;
	char *prefix;
	strname = strTableName(dict);
	prefix = prefixName(dict);
	fprintf(outf, "/* NSC table definition  for %s */\n",
		tableName(dict));
	fprintf(outf, "struct castr %s[] = { /* %d */\n", 
			nscTableName(dict), dict->objnum);
	for (f=dict->flds;f;f=f->next) {
		if (f->fldtype>0 && f->fldtype<=MAXTYPE)
			typename = nscnames[f->fldtype];
		else
			typename = nscnames[0];
		fprintf(outf, "	%-10s| fldoff(%s, %s%-16s), \"%s\",%d,0,\n",
			typename, strname, prefix,
			f->fldname, f->fldname,	f->fldcount);
	}
	fprintf(outf, "	0,0,0,0\n");
	fprintf(outf, "};\n\n");
}

void dumpUnknownTypes(f, edb)
	FILE *f;
	Empire_Database *edb;
{
	Dictionary *dict;
	Field *fld;
	for (dict=(Dictionary *)edb->dictionary; dict; dict=dict->next) 
		if (!dict->objdesc)
			describeType(f, dict);
	for (dict=(Dictionary *)edb->dictionary; dict; dict=dict->next) 
		if (!dict->objdesc)
			makeNscDefinition(f, dict);
}

void edb_dumpHeaders(f, edb)
	FILE *f;
	Empire_Database *edb;
{
	Dictionary *dict;
	Field *fld;
	for (dict=(Dictionary *)edb->dictionary; dict; dict=dict->next) 
		describeType(f, dict);
}

void edb_dumpStructs(f, edb)
	FILE *f;
	Empire_Database *edb;
{
	Dictionary *dict;
	Field *fld;
	for (dict=(Dictionary *)edb->dictionary; dict; dict=dict->next) 
		makeNscDefinition(f, dict);
}

